package range_analysis

import (
	"encoding/json"
	"eta/eta_mobile/controllers"
	"eta/eta_mobile/models"
	"eta/eta_mobile/models/data_manage"
	"eta/eta_mobile/models/system"
	"eta/eta_mobile/services/data"
	"eta/eta_mobile/services/data/data_manage_permission"
	"eta/eta_mobile/utils"
	"fmt"
	"sort"
	"time"
)

// RangeChartClassifyController 	区间分析图表
type RangeChartClassifyController struct {
	controllers.BaseAuthController
}

// ChartClassifyList
// @Title 区间分析图表分类列表
// @Description 区间分析图表分类列表接口
// @Param   IsShowMe   query   bool  false       "是否只看我的，true、false"
// @Param   ParentId   query   bool  false       "父级ID"
// @Param   Source   query   int  false       "图表类型，3：相关性，4：滚动相关性"
// @Success 200 {object} data_manage.ChartClassifyListResp
// @router /chart_classify/list [get]
func (this *RangeChartClassifyController) ChartClassifyList() {
	br := new(models.BaseResponse).Init()
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		this.Data["json"] = br
		this.ServeJSON()
	}()
	resp := new(data_manage.ChartClassifyListResp)

	// 获取当前账号的不可见指标
	//noPermissionChartIdMap := make(map[int]bool)
	//{
	//	obj := data_manage.EdbInfoNoPermissionAdmin{}
	//	confList, err := obj.GetAllChartListByAdminId(this.SysUser.AdminId)
	//	if err != nil && err.Error() != utils.ErrNoRow() {
	//		br.Msg = "获取失败"
	//		br.ErrMsg = "获取不可见指标配置数据失败，Err:" + err.Error()
	//		return
	//	}
	//	for _, v := range confList {
	//		noPermissionChartIdMap[v.ChartInfoId] = true
	//	}
	//}

	isShowMe, _ := this.GetBool("IsShowMe")
	parentId, _ := this.GetInt("ParentId")
	source, _ := this.GetInt("Source", utils.CHART_SOURCE_RANGE_ANALYSIS)

	nodeAll := make([]*data_manage.ChartClassifyItems, 0)
	// 查询分类节点
	rootList, err := data_manage.GetChartClassifyByParentId(parentId, utils.CHART_SOURCE_RANGE_ANALYSIS)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取失败"
		br.ErrMsg = "获取数据失败，Err:" + err.Error()
		return
	}

	if len(rootList) > 0 {
		permissionClassifyIdList, e := data_manage_permission.GetUserChartClassifyPermissionList(this.SysUser.AdminId, 0)
		if e != nil {
			br.Msg = "获取失败"
			br.ErrMsg = "获取已授权分类id数据失败，Err:" + e.Error()
			return
		}

		for _, v := range rootList {
			// 操作按钮权限
			v.HaveOperaAuth = data_manage_permission.CheckChartClassifyPermissionByPermissionIdList(v.IsJoinPermission, v.ChartClassifyId, permissionClassifyIdList)
			button := data.GetChartClassifyOpButton(this.SysUser, v.SysUserId, v.HaveOperaAuth)
			v.Button = button
			v.ParentId = parentId
			v.Children = make([]*data_manage.ChartClassifyItems, 0)

			nodeAll = append(nodeAll, v)
		}
	}

	// 查询图表节点, ParentId=0时说明仅查询一级目录节点
	if parentId > 0 {
		// 查询当前分类信息
		currClassify, e := data_manage.GetChartClassifyById(parentId)
		if e != nil {
			br.Msg = "获取失败"
			br.ErrMsg = "获取当前分类信息失败，Err:" + e.Error()
			return
		}

		// 获取所有有权限的指标和分类
		permissionEdbIdList, permissionClassifyIdList, e := data_manage_permission.GetUserChartAndClassifyPermissionList(this.SysUser.AdminId, 0, 0)
		if e != nil {
			br.Msg = "获取失败"
			br.ErrMsg = "获取所有有权限的指标和分类失败，Err:" + e.Error()
			return
		}

		var adminId int
		if isShowMe {
			adminId = this.SysUser.AdminId
		}

		charts, e := data_manage.GetChartInfoBySourceAndParentId(source, parentId, adminId)
		if e != nil {
			br.Msg = "获取失败"
			br.ErrMsg = fmt.Sprintf("获取图表信息失败, Err: %v", e)
			return
		}
		for _, v := range charts {
			// 操作按钮权限
			v.HaveOperaAuth = data_manage_permission.CheckChartPermissionByPermissionIdList(v.IsJoinPermission, currClassify.IsJoinPermission, v.ChartInfoId, v.ChartClassifyId, permissionEdbIdList, permissionClassifyIdList)
			button := data.GetChartOpButton(this.SysUser, v.SysUserId, v.HaveOperaAuth)
			button.AddButton = false //不管有没有权限，图表都是没有添加按钮的
			v.Button = button
			v.ParentId = parentId
			v.Children = make([]*data_manage.ChartClassifyItems, 0)

			nodeAll = append(nodeAll, v)
		}
	}

	// 整体排序
	if len(nodeAll) > 0 {
		sort.Slice(nodeAll, func(i, j int) bool {
			return nodeAll[i].Sort < nodeAll[j].Sort
		})
	}

	resp.AllNodes = nodeAll
	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resp
}

// getChartClassifyListForMe 获取我创建的图表
func getChartClassifyListForMe(adminInfo system.Admin, resp *data_manage.ChartClassifyListResp) (errMsg string, err error) {
	rootList, err := data_manage.GetChartClassifyByParentId(0, utils.CHART_SOURCE_RANGE_ANALYSIS)
	if err != nil && err.Error() != utils.ErrNoRow() {
		errMsg = "获取失败"
		return
	}

	classifyAll, err := data_manage.GetChartClassifyAll(utils.CHART_SOURCE_RANGE_ANALYSIS)
	if err != nil && err.Error() != utils.ErrNoRow() {
		errMsg = "获取失败"
		return
	}

	allChartInfo, err := data_manage.GetChartInfoByAdminId([]int{utils.CHART_SOURCE_RANGE_ANALYSIS, utils.CHART_SOURCE_RANGE_ANALYSIS}, adminInfo.AdminId)
	if err != nil && err.Error() != utils.ErrNoRow() {
		errMsg = "获取失败"
		return
	}

	chartInfoMap := make(map[int][]*data_manage.ChartClassifyItems)
	for _, v := range allChartInfo {
		chartInfoMap[v.ChartClassifyId] = append(chartInfoMap[v.ChartClassifyId], v)
	}
	rootChildMap := make(map[int][]*data_manage.ChartClassifyItems)
	for _, v := range classifyAll {
		rootChildMap[v.ParentId] = append(rootChildMap[v.ParentId], v)
		if existItems, ok := chartInfoMap[v.ChartClassifyId]; ok {
			v.Children = existItems
		} else {
			items := make([]*data_manage.ChartClassifyItems, 0)
			v.Children = items
		}
	}
	nodeAll := make([]*data_manage.ChartClassifyItems, 0)
	for _, v := range rootList {
		if existItems, ok := rootChildMap[v.ChartClassifyId]; ok {
			v.Children = existItems
		} else {
			items := make([]*data_manage.ChartClassifyItems, 0)
			v.Children = items
		}
		nodeAll = append(nodeAll, v)
	}
	resp.AllNodes = nodeAll

	return
}

// ChartClassifyItems
// @Title 获取所有区间分析图表分类接口-不包含图表
// @Description 获取所有区间分析图表分类接口-不包含图表
// @Success 200 {object} data_manage.ChartClassifyListResp
// @router /chart_classify/items [get]
func (this *RangeChartClassifyController) ChartClassifyItems() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	rootList, err := data_manage.GetChartClassifyByParentId(0, utils.CHART_SOURCE_RANGE_ANALYSIS)
	if err != nil {
		br.Msg = "获取失败"
		br.ErrMsg = "获取数据失败，Err:" + err.Error()
		return
	}

	nodeAll := make([]*data_manage.ChartClassifyItems, 0)
	for k := range rootList {
		rootNode := rootList[k]
		nodeAll = append(nodeAll, rootNode)
	}
	resp := new(data_manage.ChartClassifyListResp)
	resp.AllNodes = nodeAll
	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resp
}

// AddChartClassify
// @Title 新增区间分析图表分类
// @Description 新增区间分析图表分类接口
// @Param	request	body data_manage.AddChartClassifyReq true "type json string"
// @Success 200 Ret=200 保存成功
// @router /chart_classify/add [post]
func (this *RangeChartClassifyController) AddChartClassify() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req data_manage.AddChartClassifyReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.ChartClassifyName == "" {
		br.Msg = "请输入分类名称"
		br.IsSendEmail = false
		return
	}
	if req.ParentId < 0 {
		br.Msg = "参数错误"
		br.IsSendEmail = false
		return
	}

	// 新增图表分类
	_, err, errMsg, isSendEmail := data.AddChartClassify(req.ChartClassifyName, req.ParentId, req.Level, utils.CHART_SOURCE_RANGE_ANALYSIS, this.Lang, this.SysUser)
	if err != nil {
		br.Msg = errMsg
		br.ErrMsg = "添加分类失败,Err:" + err.Error()
		br.IsSendEmail = isSendEmail
		return
	}

	br.Ret = 200
	br.Msg = "添加成功"
	br.Success = true
}

// EditChartClassify
// @Title 修改区间分析图表分类
// @Description 修改区间分析图表分类接口
// @Param	request	body data_manage.EditChartClassifyReq true "type json string"
// @Success 200 Ret=200 修改成功
// @router /chart_classify/edit [post]
func (this *RangeChartClassifyController) EditChartClassify() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req data_manage.EditChartClassifyReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.ChartClassifyName == "" {
		br.Msg = "请输入分类名称"
		br.IsSendEmail = false
		return
	}

	if req.ChartClassifyId <= 0 {
		br.Msg = "参数错误"
		br.IsSendEmail = false
		return
	}

	// 编辑图表分类
	_, err, errMsg, isSendEmail := data.EditChartClassify(req.ChartClassifyId, utils.CHART_SOURCE_RANGE_ANALYSIS, req.ChartClassifyName, this.Lang, this.SysUser)
	if err != nil {
		br.Msg = errMsg
		br.ErrMsg = "保存分类失败,Err:" + err.Error()
		br.IsSendEmail = isSendEmail
		return
	}

	br.Ret = 200
	br.Msg = "修改成功"
	br.Success = true
	br.IsAddLog = true
}

// DeleteChartClassifyCheck
// @Title 删除图表检测接口
// @Description 删除图表检测接口
// @Param	request	body data_manage.ChartClassifyDeleteCheckResp true "type json string"
// @Success 200 Ret=200 检测成功
// @router /chart_classify/delete/check [post]
func (this *RangeChartClassifyController) DeleteChartClassifyCheck() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req data_manage.ChartClassifyDeleteCheckReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}

	if req.ChartClassifyId < 0 && req.ChartInfoId <= 0 {
		br.Msg = "参数错误"
		br.IsSendEmail = false
		return
	}
	var deleteStatus int
	var tipsMsg string
	//删除分类
	if req.ChartClassifyId > 0 && req.ChartInfoId == 0 {
		//判断区间分析图表分类下，是否含有图表
		count, err := data_manage.GetChartInfoCountByClassifyId(req.ChartClassifyId)
		if err != nil {
			br.Msg = "删除失败"
			br.ErrMsg = "分类下是否含有图表失败，Err:" + err.Error()
			return
		}

		if count > 0 {
			deleteStatus = 1
			tipsMsg = "该分类下关联图表不可删除"
		}
	}

	if deleteStatus != 1 && req.ChartInfoId == 0 {
		classifyCount, err := data_manage.GetChartClassifyCountByClassifyId(req.ChartClassifyId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "删除失败"
			br.ErrMsg = "分类下是否含有图表失败，Err:" + err.Error()
			return
		}
		if classifyCount > 0 {
			deleteStatus = 2
			tipsMsg = "确认删除当前目录及包含的子目录吗"
		}
	}
	if deleteStatus == 0 {
		tipsMsg = "可删除，进行删除操作"
	}

	resp := new(data_manage.ChartClassifyDeleteCheckResp)
	resp.DeleteStatus = deleteStatus
	resp.TipsMsg = tipsMsg
	br.Ret = 200
	br.Msg = "检测成功"
	br.Success = true
	br.Data = resp
}

// DeleteChartClassify
// @Title 删除区间分析图表分类/图表
// @Description 删除区间分析图表分类/图表接口
// @Param	request	body data_manage.DeleteChartClassifyReq true "type json string"
// @Success 200 Ret=200 删除成功
// @router /chart_classify/delete [post]
func (this *RangeChartClassifyController) DeleteChartClassify() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	var req data_manage.DeleteChartClassifyReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}

	if req.ChartClassifyId < 0 && req.ChartInfoId <= 0 {
		br.Msg = "参数错误"
		br.IsSendEmail = false
		return
	}

	//删除分类
	if req.ChartClassifyId > 0 && req.ChartInfoId == 0 {
		//判断是否含有指标
		count, err := data_manage.GetChartInfoCountByClassifyId(req.ChartClassifyId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "删除失败"
			br.ErrMsg = "判断名称是否已存在失败，Err:" + err.Error()
			return
		}

		if count > 0 {
			br.Msg = "该目录下存在关联指标，不可删除"
			br.IsSendEmail = false
			return
		}

		err = data_manage.DeleteChartClassify(req.ChartClassifyId)
		if err != nil {
			br.Msg = "删除失败"
			br.ErrMsg = "删除失败,Err:" + err.Error()
			return
		}
	}
	resp := new(data_manage.AddChartInfoResp)
	//删除图表
	if req.ChartInfoId > 0 {
		chartInfo, err := data_manage.GetChartInfoById(req.ChartInfoId)
		if err != nil {
			if err.Error() == utils.ErrNoRow() {
				br.Msg = "图表已删除，请刷新页面"
				br.ErrMsg = "指标不存在,Err:" + err.Error()
				return
			} else {
				br.Msg = "删除失败"
				br.ErrMsg = "删除失败，获取指标信息失败，Err:" + err.Error()
				return
			}
		}
		if chartInfo == nil {
			br.Msg = "图表已删除，请刷新页面"
			return
		}
		//图表操作权限
		ok := data.CheckOpChartPermission(sysUser, chartInfo.SysUserId, true)
		if !ok {
			br.Msg = "没有该图表的操作权限"
			br.ErrMsg = "没有该图表的操作权限"
			return
		}

		// 获取引用该图表的MyCharts, 用于ES删除
		var myCond string
		var myPars []interface{}
		myCond += ` AND a.chart_info_id = ? `
		myPars = append(myPars, chartInfo.ChartInfoId)
		myCharts, e := data_manage.GetMyChartListGroupByCharyInfoIdAndAdminIdByCondition(myCond, myPars)
		if e != nil {
			br.Msg = "删除失败"
			br.ErrMsg = "获取引用图表的MyChats失败, Err: " + e.Error()
			return
		}
		myIds := make([]int, 0)
		for _, m := range myCharts {
			myIds = append(myIds, m.MyChartId)
		}

		source := chartInfo.Source // 区间分析图表(滚动相关性)
		//删除图表及关联指标
		err = data_manage.DeleteChartInfoAndData(chartInfo.ChartInfoId)
		if err != nil {
			br.Msg = "删除失败"
			br.ErrMsg = "删除失败，Err:" + err.Error()
			return
		}

		// 删除图表系列
		chartSeriesOb := new(data_manage.FactorEdbSeriesChartMapping)
		seriesMappingItem, e := chartSeriesOb.GetItemByChartInfoId(chartInfo.ChartInfoId)
		if e != nil {
			if e.Error() == utils.ErrNoRow() {
			} else {
				br.Msg = "删除失败"
				br.ErrMsg = "获取图表关联失败, Err: " + e.Error()
				return
			}
		} else {
			factorSeriesOb := new(data_manage.FactorEdbSeries)
			e = factorSeriesOb.RemoveSeriesAndMappingByFactorEdbSeriesId(seriesMappingItem)
			if e != nil {
				br.Msg = "删除失败"
				br.ErrMsg = "获取图表关联失败, Err: " + e.Error()
				return
			}
		}
		//删除ES
		{
			go data.EsDeleteChartInfo(chartInfo.ChartInfoId)
			// 删除MY ETA 图表 es数据
			//go data.EsDeleteMyChartInfoByChartInfoId(chartInfo.ChartInfoId)
			go data.EsDeleteMyChartInfoByMyChartIds(myIds)
		}

		var condition string
		var pars []interface{}
		condition += " AND chart_classify_id=? AND source = ? "
		pars = append(pars, chartInfo.ChartClassifyId, source)

		condition += " AND chart_info_id>? ORDER BY create_time ASC LIMIT 1 "
		pars = append(pars, req.ChartInfoId)

		nextItem, err := data_manage.GetChartInfoByCondition(condition, pars)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "删除失败"
			br.ErrMsg = "获取下一级图库信息失败，Err:" + err.Error()
			return
		}

		if nextItem != nil {
			resp.UniqueCode = nextItem.UniqueCode
			resp.ChartInfoId = nextItem.ChartInfoId
		} else {
			var condition string
			var pars []interface{}

			condition += " AND level=1 "
			//pars = append(pars, chartInfo.ChartClassifyId)

			condition += " AND chart_classify_id>? ORDER BY chart_classify_id ASC LIMIT 1 "
			pars = append(pars, chartInfo.ChartClassifyId)

			classifyItem, err := data_manage.GetChartClassifyByCondition(condition, pars)
			if err != nil && err.Error() != utils.ErrNoRow() {
				br.Msg = "删除失败"
				br.ErrMsg = "获取下一级图库分类信息失败，Err:" + err.Error()
				return
			}
			if classifyItem != nil {
				nextItem, err = data_manage.GetNextChartInfo(chartInfo.ChartClassifyId)
				if err != nil && err.Error() != utils.ErrNoRow() {
					br.Msg = "删除失败"
					br.ErrMsg = "获取下一级图库信息失败，Err:" + err.Error()
					return
				}
				if nextItem != nil {
					resp.UniqueCode = nextItem.UniqueCode
					resp.ChartInfoId = nextItem.ChartInfoId
				}
			}
		}
		//新增操作日志
		{
			chartLog := new(data_manage.ChartInfoLog)
			chartLog.ChartName = chartInfo.ChartName
			chartLog.ChartInfoId = req.ChartInfoId
			chartLog.ChartClassifyId = chartInfo.ChartClassifyId
			chartLog.SysUserId = sysUser.AdminId
			chartLog.SysUserRealName = sysUser.RealName
			chartLog.UniqueCode = chartInfo.UniqueCode
			chartLog.CreateTime = time.Now()
			chartLog.Content = string(this.Ctx.Input.RequestBody)
			chartLog.Status = "删除图表"
			chartLog.Method = this.Ctx.Input.URI()
			go data_manage.AddChartInfoLog(chartLog)
		}
	}
	br.Ret = 200
	br.Msg = "删除成功"
	br.Success = true
	br.Data = resp
	br.IsAddLog = true
}

// ClassifyTree
// @Title 多层分类列表树
// @Description 多层分类列表树
// @Success 200 {object} data_manage.ChartClassifyListResp
// @router /chart_classify/tree [get]
func (this *RangeChartClassifyController) ClassifyTree() {
	br := new(models.BaseResponse).Init()
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		this.Data["json"] = br
		this.ServeJSON()
	}()

	allList, err := data_manage.GetChartClassifyAllBySource(utils.CHART_SOURCE_RANGE_ANALYSIS)
	if err != nil {
		br.Msg = "获取失败"
		br.ErrMsg = "获取所有分类失败, Err:" + err.Error()
		return
	}
	nodeAll := make([]*data_manage.ChartClassifyItems, 0)

	if len(allList) > 0 {
		// 已授权分类id
		permissionClassifyIdList, e := data_manage_permission.GetUserChartClassifyPermissionList(this.SysUser.AdminId, 0)
		if e != nil {
			br.Msg = "获取失败"
			br.ErrMsg = "获取已授权分类id数据失败，Err:" + e.Error()
			return
		}

		for k, v := range allList {
			// 数据权限
			v.HaveOperaAuth = data_manage_permission.CheckChartClassifyPermissionByPermissionIdList(v.IsJoinPermission, v.ChartClassifyId, permissionClassifyIdList)
			// 按钮权限
			button := data.GetChartClassifyOpButton(this.SysUser, v.SysUserId, v.HaveOperaAuth)
			allList[k].Button = button
		}

		nodeAll = data.GetChartClassifyTreeRecursive(allList, 0)
		//根据sort值排序
		sort.Slice(nodeAll, func(i, j int) bool {
			return nodeAll[i].Sort < nodeAll[j].Sort
		})
	}

	language := `CN`
	// 显示的语言
	{
		configDetail, _ := system.GetConfigDetailByCode(this.SysUser.AdminId, system.ChartLanguageVar)
		if configDetail != nil {
			language = configDetail.ConfigValue
		} else {
			configDetail, _ = system.GetDefaultConfigDetailByCode(system.ChartLanguageVar)
			if configDetail != nil {
				language = configDetail.ConfigValue
			}
		}
	}

	// 是否允许添加一级分类
	canOpClassify := true
	button := data.GetChartClassifyOpButton(this.SysUser, 0, true)
	if !button.AddButton {
		canOpClassify = false
	}

	resp := new(data_manage.ChartClassifyListResp)
	resp.AllNodes = nodeAll
	resp.Language = language
	resp.CanOpClassify = canOpClassify
	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resp
}
